#
#    Uncomplicated VM Builder
#    Copyright (C) 2007-2009 Canonical Ltd.
#    
#    See AUTHORS for list of contributors
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License version 3, as
#    published by the Free Software Foundation.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
#    Distro super class

import logging
import os

from   VMBuilder.util    import run_cmd, call_hooks
import VMBuilder.plugins

class Context(VMBuilder.plugins.Plugin):
    def __init__(self):
        self._config = {}
        super(Context, self).__init__(self)
        self.plugins = [plugin_class(self) for plugin_class in self.plugin_classes]
        self.plugins.sort(key=lambda x:x.priority)
        self._cleanup_cbs = []
        self.hooks = {}
        self.template_dirs = [os.path.expanduser('~/.vmbuilder/%s'),
                              os.path.dirname(__file__) + '/plugins/%s/templates',
                              '/etc/vmbuilder/%s']
        self.overwrite = False

    # Cleanup 
    def cleanup(self):
        logging.info("Cleaning up")
        while len(self._cleanup_cbs) > 0:
            self._cleanup_cbs.pop(0)()

    def add_clean_cb(self, cb):
        self._cleanup_cbs.insert(0, cb)

    def add_clean_cmd(self, *argv, **kwargs):
        cb = lambda : run_cmd(*argv, **kwargs)
        self.add_clean_cb(cb)
        return cb

    def cancel_cleanup(self, cb):
        try:
            self._cleanup_cbs.remove(cb)
        except ValueError:
            # Wasn't in there. No worries.
            pass

    # Hooks
    def register_hook(self, hook_name, func):
        self.hooks[hook_name] = self.hooks.get(hook_name, []) + [func]

    def call_hooks(self, *args, **kwargs):
        try:
            call_hooks(self, *args, **kwargs)
        except Exception:
            self.cleanup()
            raise

class Distro(Context):
    def __init__(self):
        self.plugin_classes = VMBuilder._distro_plugins
        super(Distro, self).__init__()

    def set_chroot_dir(self, chroot_dir):
        self.chroot_dir = chroot_dir 

    def build_chroot(self):
        self.call_hooks('preflight_check')
        self.call_hooks('set_defaults')
        self.call_hooks('bootstrap')
        self.call_hooks('configure_os')
	self.cleanup()
        
    def has_xen_support(self):
        """Install the distro into destdir"""
        raise NotImplemented('Distro subclasses need to implement the has_xen_support method')
    
    def install(self, destdir):
        """Install the distro into destdir"""
        raise NotImplemented('Distro subclasses need to implement the install method')

    def post_mount(self, fs):
        """Called each time a filesystem is mounted to let the distro add things to the filesystem"""

    def install_vmbuilder_log(self, logfile):
        """Let the distro copy the install logfile to the guest"""



import unittest

import VMBuilder.plugins
from   VMBuilder.exception import VMBuilderException

class TestPluginsSettings(unittest.TestCase):
    class VM(VMBuilder.plugins.Plugin):
        def __init__(self, *args, **kwargs):
            self._config = {}
            self.context = self

    class TestPlugin(VMBuilder.plugins.Plugin):
        pass

    def setUp(self):
        self.vm = self.VM()
        self.plugin = self.TestPlugin(self.vm)
        self.i = 0

    def test_add_setting_group_and_setting(self):
        setting_group = self.plugin.setting_group('Test Setting Group')
        self.assertTrue(setting_group in self.plugin._setting_groups, "Setting not added correctly to plugin's registry of setting groups.")

        setting_group.add_setting('testsetting')
        self.assertEqual(self.vm.get_setting('testsetting'), None, "Setting's default value is not None.")

        self.vm.set_setting_default('testsetting', 'newdefault')
        self.assertEqual(self.vm.get_setting('testsetting'), 'newdefault', "Setting does not return custom default value when no value is set.")
        self.assertEqual(self.vm.get_setting_default('testsetting'), 'newdefault', "Setting does not return custom default value through get_setting_default().")

        self.vm.set_setting('testsetting', 'foo')
        self.assertEqual(self.vm.get_setting('testsetting'), 'foo', "Setting does not return set value.")

        self.vm.set_setting_default('testsetting', 'newerdefault')
        self.assertEqual(self.vm.get_setting('testsetting'), 'foo', "Setting does not return set value after setting new default value.")

    def test_invalid_type_raises_exception(self):
        setting_group = self.plugin.setting_group('Test Setting Group')
        self.assertRaises(VMBuilderException, setting_group.add_setting, 'oddsetting', type='odd')

    def test_valid_options(self):
        setting_group = self.plugin.setting_group('Test Setting Group')

        setting_group.add_setting('strsetting')
        self.assertRaises(VMBuilderException, self.vm.set_setting_valid_options, 'strsetting', '')
        self.vm.set_setting_valid_options('strsetting', ['foo', 'bar'])
        self.assertEqual(self.vm.get_setting_valid_options('strsetting'), ['foo', 'bar'])
        self.vm.set_setting('strsetting', 'foo')
        self.assertRaises(VMBuilderException, self.vm.set_setting, 'strsetting', 'baz')
        self.vm.set_setting_valid_options('strsetting', None)
        self.vm.set_setting('strsetting', 'baz')

    def test_invalid_type_setting_raises_exception(self):
        setting_group = self.plugin.setting_group('Test Setting Group')

        test_table = [{ 'type' : 'str',
                        'good' : [''],
                        'fuzzy': [''],
                        'bad'  : [0, True, ['foo']]
                      },
                      { 'type' : 'int',
                        'good' : [0],
                        'fuzzy': [('0', 0), ('34', 34), (0, 0), (34, 34)],
                        'bad'  : ['', '0', True, ['foo']]
                      },
                      { 'type' : 'bool',
                        'good' : [True],
                        'fuzzy': [(True, True), ('tRuE', True), ('oN', True), ('yEs', True), ('1', True),
                                  (False, False), ('fAlSe', False), ('oFf', False), ('nO', False), ('0', False) ],
                        'bad'  : ['', 0, '0', ['foo'], '1']
                      },
                      { 'type' : 'list',
                        'good' : [['foo']],
                        'fuzzy': [('main    ,        universe,multiverse', ['main', 'universe', 'multiverse']),
                                  ('main:universe:multiverse', ['main', 'universe', 'multiverse']),
                                  ('''main:
                                  universe:multiverse''', ['main', 'universe', 'multiverse']),
                                  ('',  [])],
                        'bad'  : [True, '', 0, '0']
                      }]

        def get_new_setting(setting_type):
            setting_name = '%ssetting%d' % (setting_type, self.i)
            self.i += 1
            setting_group.add_setting(setting_name, type=setting_type)
            return setting_name

        def try_bad_setting(setting_type, bad, setter):
            setting_name = get_new_setting(setting_type)
            self.assertRaises(VMBuilderException, setter, setting_name, bad)

        def try_good_setting(setting_type, good, getter, setter):
            setting_name = get_new_setting(setting_type)
            if type(good) == tuple:
                in_value, out_value = good
            else:
                in_value, out_value = good, good

#            print setting_name, in_value
            setter(setting_name, in_value)
            self.assertEqual(getter(setting_name), out_value)

        for setting_type in test_table:
            for good in setting_type['good']:
                try_good_setting(setting_type['type'], good, self.vm.get_setting, self.vm.set_setting)
                try_good_setting(setting_type['type'], good, self.vm.get_setting, self.vm.set_setting_default)
                try_good_setting(setting_type['type'], good, self.vm.get_setting_default, self.vm.set_setting_default)
                try_good_setting(setting_type['type'], good, self.vm.get_setting, self.vm.set_setting_fuzzy)
            for fuzzy in setting_type['fuzzy']:
                try_good_setting(setting_type['type'], fuzzy, self.vm.get_setting, self.vm.set_setting_fuzzy)
            for bad in setting_type['bad']:
                try_bad_setting(setting_type['type'], bad, self.vm.set_setting)
                try_bad_setting(setting_type['type'], bad, self.vm.set_setting_default)

    def test_set_setting_raises_exception_on_invalid_setting(self):
        self.assertRaises(VMBuilderException, self.vm.set_setting_default, 'testsetting', 'newdefault')

    def test_add_setting(self):
        setting_group = self.plugin.setting_group('Test Setting Group')



#
#    Uncomplicated VM Builder
#    Copyright (C) 2007-2009 Canonical Ltd.
#    
#    See AUTHORS for list of contributors
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License version 3, as
#    published by the Free Software Foundation.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
#    Logging

import logging
import os
import tempfile

format = '%(asctime)s %(levelname)-8s: %(message)s'

fd, logfile = tempfile.mkstemp()

# Log everything to the logfile
logging.basicConfig(format=format, level=logging.DEBUG, datefmt='%Y-%m-%d %H:%M', stream=os.fdopen(fd, 'a+'), filemode='w')

console = logging.StreamHandler()
console.setLevel(logging.INFO)
console.setFormatter(logging.Formatter(format))
logging.getLogger('').addHandler(console)

logging.info("logging to file: {}".format(logfile))



#
#    Uncomplicated VM Builder
#    Copyright (C) 2007-2010 Canonical Ltd.
#
#    See AUTHORS for list of contributors
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License version 3, as
#    published by the Free Software Foundation.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
import logging
import os
import shutil
import stat
import VMBuilder
from   VMBuilder           import register_distro, Distro
from   VMBuilder.util      import run_cmd
from   VMBuilder.exception import VMBuilderUserError, VMBuilderException

class Ubuntu(Distro):
    name = 'Ubuntu'
    arg = 'ubuntu'
    suites = ['dapper', 'gutsy', 'hardy', 'intrepid', 'jaunty', 
              'karmic', 'lucid', 'maverick', 'natty', 'oneiric',
              'precise', 'quantal', 'raring', 'saucy', 'trusty' ]

    # Maps host arch to valid guest archs
    valid_archs = { 'amd64' : ['amd64', 'i386', 'lpia' ],
                    'i386' : [ 'i386', 'lpia' ],
                    'lpia' : [ 'i386', 'lpia' ] }

    xen_kernel = ''

    def register_options(self):
        group = self.setting_group('Package options')
        group.add_setting('addpkg', type='list', metavar='PKG', help='Install PKG into the guest (can be specified multiple times).')
        group.add_setting('removepkg', type='list', metavar='PKG', help='Remove PKG from the guest (can be specified multiple times)')
        group.add_setting('seedfile', metavar="SEEDFILE", help='Seed the debconf database with the contents of this seed file before installing packages')

        group = self.setting_group('General OS options')
        self.host_arch = run_cmd('dpkg', '--print-architecture').rstrip()
        group.add_setting('arch', extra_args=['-a'], default=self.host_arch, help='Specify the target architecture.  Valid options: amd64 i386 lpia (defaults to host arch)')
        group.add_setting('hostname', default='ubuntu', help='Set NAME as the hostname of the guest. Default: ubuntu. Also uses this name as the VM name.')

        group = self.setting_group('Installation options')
        group.add_setting('suite', default='lucid', help='Suite to install. Valid options: %s [default: %%default]' % ' '.join(self.suites))
        group.add_setting('flavour', extra_args=['--kernel-flavour'], help='Kernel flavour to use. Default and valid options depend on architecture and suite')
        group.add_setting('variant', metavar='VARIANT', help='Passed to debootstrap --variant flag; use minbase, buildd, or fakechroot.')
        group.add_setting('debootstrap-tarball', metavar='FILE', help='Passed to debootstrap --unpack-tarball flag.')
        group.add_setting('iso', metavar='PATH', help='Use an iso image as the source for installation of file. Full path to the iso must be provided. If --mirror is also provided, it will be used in the final sources.list of the vm.  This requires suite and kernel parameter to match what is available on the iso, obviously.')
        group.add_setting('mirror', metavar='URL', help='Use Ubuntu mirror at URL instead of the default, which is http://archive.ubuntu.com/ubuntu for official arches and http://ports.ubuntu.com/ubuntu-ports otherwise')
        group.add_setting('proxy', metavar='URL', help='Use proxy at URL for cached packages')
        group.add_setting('install-mirror', metavar='URL', help='Use Ubuntu mirror at URL for the installation only. Apt\'s sources.list will still use default or URL set by --mirror')
        group.add_setting('security-mirror', metavar='URL', help='Use Ubuntu security mirror at URL instead of the default, which is http://security.ubuntu.com/ubuntu for official arches and http://ports.ubuntu.com/ubuntu-ports otherwise.')
        group.add_setting('install-security-mirror', metavar='URL', help='Use the security mirror at URL for installation only. Apt\'s sources.list will still use default or URL set by --security-mirror')
        group.add_setting('components', type='list', metavar='COMPS', help='A comma seperated list of distro components to include (e.g. main,universe).')
        group.add_setting('ppa', metavar='PPA', type='list', help='Add ppa belonging to PPA to the vm\'s sources.list.')
        group.add_setting('lang', metavar='LANG', default=get_locale(), help='Set the locale to LANG [default: %default]')
        group.add_setting('timezone', metavar='TZ', default='UTC', help='Set the timezone to TZ in the vm. [default: %default]')

        group = self.setting_group('Settings for the initial user')
        group.add_setting('user', default='ubuntu', help='Username of initial user [default: %default]')
        group.add_setting('name', default='Ubuntu', help='Full name of initial user [default: %default]')
        group.add_setting('pass', default='ubuntu', help='Password of initial user [default: %default]')
        group.add_setting('rootpass', help='Initial root password (WARNING: this has strong security implications).')
        group.add_setting('uid', type='int', help='Initial UID value.')
        group.add_setting('gid', help='Initial GID value.')
        group.add_setting('lock-user', type='bool', default=False, help='Lock the initial user [default: %default]')

        group = self.setting_group('Other options')
        group.add_setting('ssh-key', metavar='PATH', help='Add PATH to root\'s ~/.ssh/authorized_keys (WARNING: this has strong security implications).')
        group.add_setting('ssh-user-key', help='Add PATH to the user\'s ~/.ssh/authorized_keys.')
        group.add_setting('manifest', metavar='PATH', help='If passed, a manifest will be written to PATH')

    def set_defaults(self):
        arch = self.get_setting('arch')

        if arch == 'lpia':
            self.set_setting_default('mirror', 'http://ports.ubuntu.com/ubuntu-ports')
            self.set_setting_default('security-mirror', 'http://ports.ubuntu.com/ubuntu-ports')
        else:
            self.set_setting_default('mirror', 'http://archive.ubuntu.com/ubuntu')
            self.set_setting_default('security-mirror', 'http://security.ubuntu.com/ubuntu')

        self.set_setting_default('components',  ['main', 'restricted', 'universe'])

    def preflight_check(self):
        """While not all of these are strictly checks, their failure would inevitably
        lead to failure, and since we can check them before we start setting up disk
        and whatnot, we might as well go ahead an do this now."""

        suite = self.get_setting('suite') 
        if not suite in self.suites:
            raise VMBuilderUserError('Invalid suite: "%s". Valid suites are: %s' % (suite, ' '.join(self.suites)))
        
        modname = 'VMBuilder.plugins.ubuntu.%s' % (suite, )
        mod = __import__(modname, fromlist=[suite])
        self.suite = getattr(mod, suite.capitalize())(self)

        arch = self.get_setting('arch') 
        if arch not in self.valid_archs[self.host_arch] or  \
            not self.suite.check_arch_validity(arch):
            raise VMBuilderUserError('%s is not a valid architecture. Valid architectures are: %s' % (arch,
                                                                                                      ' '.join(self.valid_archs[self.host_arch])))

        components = self.get_setting('components')
        if not components:
            self.set_config_value_list = ['main', 'restricted', 'universe']
        else:
            if type(components) is str:
                self.vm.components = self.vm.components.split(',')

        self.context.virtio_net = self.use_virtio_net()

        # check if the seedfile exists if one is to be used
        seedfile = self.context.get_setting('seedfile')
        if seedfile and not os.path.exists(seedfile):
            raise VMBuilderUserError("Seedfile '%s' does not exist" % seedfile)

        lang = self.get_setting('lang')

# FIXME
#        if getattr(self.vm, 'ec2', False):
#            self.get_ec2_kernel()
#            self.get_ec2_ramdisk()
#            self.apply_ec2_settings()

    def bootstrap(self):
        self.suite.debootstrap()
        self.suite.pre_install()

    def configure_os(self):
        self.suite.install_apt_proxy()
        self.suite.install_sources_list()
        self.suite.create_devices()
        self.suite.prevent_daemons_starting()
        self.suite.mount_dev_proc()
        self.suite.install_extras()
        self.suite.create_initial_user()
        self.suite.install_authorized_keys()
        self.suite.set_timezone()
        self.suite.set_locale()
        self.suite.update()
        self.suite.install_sources_list(final=True)
        self.suite.run_in_target('apt-get', 'clean');
        self.suite.unmount_volatile()
        self.suite.unmount_proc()
        self.suite.unmount_dev_pts()
        self.suite.unmount_dev()
        self.suite.unprevent_daemons_starting()
        self.suite.create_manifest()

    def configure_networking(self, nics):
        self.suite.config_host_and_domainname()
        self.suite.config_interfaces(nics)

    def configure_mounting(self, disks, filesystems):
        self.suite.install_fstab(disks, filesystems)

    def install(self, destdir):
        self.destdir = destdir
        self.suite.install(destdir)

    def install_vmbuilder_log(self, logfile, rootdir):
        self.suite.install_vmbuilder_log(logfile, rootdir)

    def post_mount(self, fs):
        self.suite.post_mount(fs)

    def use_virtio_net(self):
        return self.suite.virtio_net

    def install_bootloader_cleanup(self, chroot_dir):
        self.context.cancel_cleanup(self.install_bootloader_cleanup)
        tmpdir = '%s/tmp/vmbuilder-grub' % chroot_dir
        for disk in os.listdir(tmpdir):
            if disk != 'device.map':
                run_cmd('umount', os.path.join(tmpdir, disk))
        shutil.rmtree(tmpdir)

    def install_kernel(self, destdir):
        self.suite.install_kernel(destdir)

    def install_bootloader(self, chroot_dir, disks):
        root_dev = VMBuilder.disk.bootpart(disks).get_grub_id()

        tmpdir = '/tmp/vmbuilder-grub'
        os.makedirs('%s%s' % (chroot_dir, tmpdir))
        self.context.add_clean_cb(self.install_bootloader_cleanup)
        devmapfile = os.path.join(tmpdir, 'device.map')
        devmap = open('%s%s' % (chroot_dir, devmapfile), 'w')
        for (disk, id) in zip(disks, range(len(disks))):
            new_filename = os.path.join(tmpdir, os.path.basename(disk.filename))
            open('%s%s' % (chroot_dir, new_filename), 'w').close()
            run_cmd('mount', '--bind', disk.filename, '%s%s' % (chroot_dir, new_filename))
            st = os.stat(disk.filename)
            if stat.S_ISBLK(st.st_mode):
                for (part, part_id) in zip(disk.partitions, range(len(disk.partitions))):
                    part_mountpnt = '%s%s%d' % (chroot_dir, new_filename, part_id+1)
                    open(part_mountpnt, 'w').close()
                    run_cmd('mount', '--bind', part.filename, part_mountpnt)
            devmap.write("(hd%d) %s\n" % (id, new_filename))
        devmap.close()
        run_cmd('cat', '%s%s' % (chroot_dir, devmapfile))
        self.suite.install_grub(chroot_dir)
        self.run_in_target('grub', '--device-map=%s' % devmapfile, '--batch',  stdin='''root %s
setup (hd0)
EOT''' % root_dev) 
        self.suite.install_menu_lst(disks)
        self.install_bootloader_cleanup(chroot_dir)

    def xen_kernel_version(self):
        if self.suite.xen_kernel_flavour:
            # if this is ec2, do not call rmadison.
            # this could be replaced with a method to get most recent
            # stable kernel, but really, this is not used at all for ec2
            if hasattr(self.context, 'ec2') and self.context.ec2:
                logging.debug("selecting ec2 kernel")
                self.xen_kernel = "2.6.ec2-kernel"
                return self.xen_kernel
            if not self.xen_kernel:
                rmad = run_cmd('rmadison', 'linux-image-%s' % self.suite.xen_kernel_flavour)
                version = ['0', '0','0', '0']

                for line in rmad.splitlines():
                    sline = line.split('|')

                    if sline[2].strip().startswith(self.context.get_setting('suite')):
                        vt = sline[1].strip().split('.')
                        for i in range(4):
                            if int(vt[i]) > int(version[i]):
                                version = vt
                                break

                if version[0] == '0':
                    raise VMBuilderException('Something is wrong, no valid xen kernel for the suite %s found by rmadison' % self.context.suite)

                self.xen_kernel = '%s.%s.%s-%s' % (version[0],version[1],version[2],version[3])
            return self.xen_kernel
        else:
            raise VMBuilderUserError('There is no valid xen kernel for the suite selected.')

    def xen_kernel_initrd_path(self, which):
        path = '/boot/%s-%s-%s' % (which, self.xen_kernel_version(), self.suite.xen_kernel_flavour)
        return path

    def xen_kernel_path(self):
        return self.xen_kernel_initrd_path('kernel')

    def xen_ramdisk_path(self):
        return self.xen_kernel_initrd_path('ramdisk')

    def get_ec2_kernel(self):
        if self.suite.ec2_kernel_info:
            return self.suite.ec2_kernel_info[self.context.arch]
        else:
            raise VMBuilderUserError('EC2 is not supported for the suite selected')

    def get_ec2_ramdisk(self):
        if self.suite.ec2_ramdisk_info:
            return self.suite.ec2_ramdisk_info[self.context.arch]
        else:
            raise VMBuilderUserError('EC2 is not supported for the suite selected')

    def disable_hwclock_access(self):
        return self.suite.disable_hwclock_access()

    def apply_ec2_settings(self):
        return self.suite.apply_ec2_settings()

    def has_256_bit_inode_ext3_support(self):
        return self.suite.has_256_bit_inode_ext3_support()

    def preferred_filesystem(self):
        return self.suite.preferred_filesystem

def get_locale():
    lang = os.getenv('LANG')
    if lang is None:
        return 'C'
    # People's $LANG looks different since lucid, but locale-gen still
    # wants the old format.
    if lang.endswith('utf8'):
        return lang[:-4] + 'UTF-8'
    return lang

register_distro(Ubuntu)



#
#    Uncomplicated VM Builder
#    Copyright (C) 2007-2009 Canonical Ltd.
#    
#    See AUTHORS for list of contributors
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License version 3, as
#    published by the Free Software Foundation.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
from   VMBuilder import register_hypervisor, Hypervisor
import VMBuilder
import VMBuilder.hypervisor
import os
import os.path
import stat
from shutil import move
from math import floor

class VMWare(Hypervisor):
    filetype = 'vmdk'
    preferred_storage = VMBuilder.hypervisor.STORAGE_DISK_IMAGE
    needs_bootloader = True
    vmxtemplate = 'vmware'

    def register_options(self):
        group = self.setting_group('VM settings')
        group.add_setting('mem', extra_args=['-m'], default='128', help='Assign MEM megabytes of memory to the guest vm. [default: %default]')
        group.add_setting('cpus', type='int', default=1, help='Assign NUM cpus to the guest vm. [default: %default]')

    def convert(self, disks, destdir):
        self.imgs = []
        for disk in self.get_disks():
            img_path = disk.convert(destdir, self.filetype)
            self.imgs.append(img_path)
            self.call_hooks('fix_ownership', img_path)

    def get_disks(self):
        return self.disks

    def deploy(self, destdir):
        mem = self.context.get_setting('mem')
        cpus = self.context.get_setting('cpus')
        hostname = self.context.distro.get_setting('hostname')
        arch = self.context.distro.get_setting('arch')
        mac = self.context.get_setting('mac')
        vmdesc = VMBuilder.util.render_template('vmware',
                                                self.context,
                                                self.vmxtemplate,
                                                { 'disks' : self.get_disks(),
                                                  'vmhwversion' : self.vmhwversion,
                                                  'mem' : mem,
                                                  'numvcpus' : cpus,
                                                  'hostname' : hostname,
                                                  'arch' : arch,
                                                  'mac' : mac,
                                                  'guestos' : (arch == 'amd64' and 'ubuntu-64' or 'ubuntu') })

        vmx = '%s/%s.vmx' % (destdir, hostname)
        fp = open(vmx, 'w')
        fp.write(vmdesc)
        fp.close()
        os.chmod(vmx, stat.S_IRWXU | stat.S_IRWXU | stat.S_IROTH | stat.S_IXOTH)
        self.call_hooks('fix_ownership', vmx)

class VMWareWorkstation6(VMWare):
    name = 'VMWare Workstation 6'
    arg = 'vmw6'
    vmhwversion = 6

class VMWareServer(VMWare):
    name = 'VMWare Server'
    arg = 'vmserver'
    vmhwversion = 4

class VMWareEsxi(VMWare):
    name = 'VMWare ESXi'
    arg = 'esxi'
    vmhwversion = 4
    adaptertype = 'lsilogic' # lsilogic | buslogic, ide is not supported by ESXi
    vmxtemplate = 'esxi.vmx'

    vmdks = [] # vmdk filenames used when deploying vmx file

    def convert(self, disks, destdir):
        self.imgs = []
        for disk in disks:

            # Move raw image to <imagename>-flat.vmdk
            diskfilename = os.path.basename(disk.filename)
            if '.' in diskfilename:
                diskfilename = diskfilename[:diskfilename.rindex('.')]

            flat = '%s/%s-flat.vmdk' % (destdir, diskfilename)
            self.vmdks.append(diskfilename)

            move(disk.filename, flat)

            self.call_hooks('fix_ownership', flat)

            # Create disk descriptor file
            sectorTotal = disk.size * 2048
            sector = int(floor(sectorTotal / 16065)) # pseudo geometry

            diskdescriptor = VMBuilder.util.render_template('vmware', self.context, 'flat.vmdk',  { 'adaptertype' : self.adaptertype, 'sectors' : sector, 'diskname' : os.path.basename(flat), 'disksize' : sectorTotal })
            vmdk = '%s/%s.vmdk' % (destdir, diskfilename)

            fp = open(vmdk, 'w')
            fp.write(diskdescriptor)
            fp.close()
            os.chmod(vmdk, stat.S_IRWXU | stat.S_IRWXU | stat.S_IROTH | stat.S_IXOTH)

            self.call_hooks('fix_ownership', vmdk)

    def get_disks(self):
        return self.vmdks

register_hypervisor(VMWareServer)
register_hypervisor(VMWareWorkstation6)
register_hypervisor(VMWareEsxi)



import unittest

import VMBuilder
from VMBuilder.util import run_cmd

class TestUtils(unittest.TestCase):
    def test_run_cmd(self):
        self.assertTrue("foobarbaztest" in run_cmd("env", env={'foobarbaztest' : 'bar' }))





#!/usr/bin/env python
from distutils.core import setup
import VMBuilder.plugins
from glob import glob
import os.path
import subprocess

if os.path.exists('.bzr'):
    try:
        o = subprocess.Popen(('bzr','version-info', '--python'), stdout=subprocess.PIPE).stdout
        f = open('VMBuilder/vcsversion.py', 'w')
        f.write(o.read())
        f.close()
        o.close()
    except Exception, e:
        print repr(e)

setup(name='VMBuilder',
      version='0.12.4',
      description='Uncomplicated VM Builder',
      author='Soren Hansen',
      author_email='soren@ubuntu.com',
      url='http://launchpad.net/vmbuilder/',
      packages=['VMBuilder', 'VMBuilder.tests', 'VMBuilder.plugins', 'VMBuilder.contrib'] + VMBuilder.plugins.find_plugins(),
      data_files=[('/etc/vmbuilder/%s' % (pkg,), glob('VMBuilder/plugins/%s/templates/*' % (pkg,))) for pkg in [p.split('.')[-1] for p in VMBuilder.plugins.find_plugins()]],
      scripts=['vmbuilder'],
      )



#
#    Uncomplicated VM Builder
#    Copyright (C) 2007-2009 Canonical Ltd.
#    
#    See AUTHORS for list of contributors
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License version 3, as
#    published by the Free Software Foundation.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
#    Hypervisor super class

import logging
import os
import VMBuilder.distro
import VMBuilder.disk
from   VMBuilder.util    import run_cmd, tmpdir

STORAGE_DISK_IMAGE = 0
STORAGE_FS_IMAGE = 1

class Hypervisor(VMBuilder.distro.Context):
    preferred_storage = STORAGE_DISK_IMAGE

    def __init__(self, distro):
        self.plugin_classes = VMBuilder._hypervisor_plugins
        super(Hypervisor, self).__init__()
        self.plugins += [distro]
        self.distro = distro
        self.filesystems = []
        self.disks = []
        self.nics = []

    def add_filesystem(self, *args, **kwargs):
        """Adds a filesystem to the virtual machine"""
        from VMBuilder.disk import Filesystem

        fs = Filesystem(self, *args, **kwargs)
        self.filesystems.append(fs)
        return fs

    def add_disk(self, *args, **kwargs):
        """Adds a disk image to the virtual machine"""
        from VMBuilder.disk import Disk

        disk = Disk(self, *args, **kwargs)
        self.disks.append(disk)
        return disk

    def install_os(self):
        self.nics = [self.NIC()]
        self.call_hooks('preflight_check')
        self.call_hooks('configure_networking', self.nics)
        self.call_hooks('configure_mounting', self.disks, self.filesystems)

        self.chroot_dir = tmpdir()
        self.call_hooks('mount_partitions', self.chroot_dir)
        run_cmd('rsync', '-aHA', '%s/' % self.distro.chroot_dir, self.chroot_dir)
        self.distro.set_chroot_dir(self.chroot_dir)
        if self.needs_bootloader:
            self.call_hooks('install_bootloader', self.chroot_dir, self.disks)
        self.call_hooks('install_kernel', self.chroot_dir)
        self.distro.call_hooks('post_install')
        self.call_hooks('unmount_partitions')
        os.rmdir(self.chroot_dir)

    def finalise(self, destdir):
        self.call_hooks('convert', 
                        self.preferred_storage == STORAGE_DISK_IMAGE and self.disks or self.filesystems,
                        destdir)
        self.call_hooks('deploy', destdir)

    def mount_partitions(self, mntdir):
        """Mounts all the vm's partitions and filesystems below .rootmnt"""
        logging.info('Mounting target filesystems')
        for fs in self.filesystems:
            fs.create()
            fs.mkfs()
        for disk in self.disks:
            disk.create()
            disk.partition()
            disk.map_partitions()
            disk.mkfs()
        fss = VMBuilder.disk.get_ordered_filesystems(self)
        for fs in fss:
            fs.mount(mntdir)
            self.distro.post_mount(fs)

    def unmount_partitions(self):
        """Unmounts all the vm's partitions and filesystems"""
        logging.info('Unmounting target filesystem')
        fss = VMBuilder.disk.get_ordered_filesystems(self)
        fss.reverse()
        for fs in fss:
            fs.umount()
        for disk in self.disks:
            disk.unmap()

    def convert_disks(self, disks, destdir):
        for disk in disks:
            disk.convert(destdir, self.filetype)

    class NIC(object):
        def __init__(self, type='dhcp', ip=None, network=None, netmask=None,
                           broadcast=None, dns=None, gateway=None):
            self.type = type
            self.ip = ip
            self.network = network
            self.netmask = netmask
            self.broadcast = broadcast
            self.dns = dns
            self.gateway = gateway




#
#    Uncomplicated VM Builder
#    Copyright (C) 2007-2009 Canonical Ltd.
#    
#    See AUTHORS for list of contributors
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License version 3, as
#    published by the Free Software Foundation.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
import VMBuilder.disk as disk
from   VMBuilder.util import run_cmd
from   VMBuilder.plugins.ubuntu.hardy import Hardy

class Intrepid(Hardy):
    valid_flavours = { 'i386' :  ['386', 'generic', 'server', 'virtual'],
                       'amd64' : ['generic', 'server', 'virtual'],
                       'lpia'  : ['lpia', 'lpiacompat'] }
    default_flavour = { 'i386' : 'virtual', 'amd64' : 'virtual', 'lpia' : 'lpia' }
    xen_kernel_flavour = 'virtual'
    ec2_kernel_info = { 'i386' : 'aki-714daa18', 'amd64' : 'aki-4f4daa26' }
    ec2_ramdisk_info = { 'i386': 'ari-7e4daa17', 'amd64' : 'ari-4c4daa25' }

    def install_ec2(self):
# workaround for policy bug on ubuntu-server. (see bug #275432)
        self.run_in_target('apt-get', '--force-yes', '-y', 'install', 'policykit')
        self.run_in_target('apt-get', '--force-yes', '-y', 'install', 'server^')
        self.install_from_template('/etc/update-motd.d/51_update-motd', '51_update-motd')
        self.run_in_target('chmod', '755', '/etc/update-motd.d/51_update-motd')
        self.install_from_template('/etc/ec2-init/is-compat-env', 'is-compat-env')

    def mangle_grub_menu_lst(self, disks):
        rootdev = disk.rootpart(disks)
        bootdev = disk.bootpart(disks)
        run_cmd('sed', '-ie', 's/^# kopt=root=\([^ ]*\)\(.*\)/# kopt=root=UUID=%s\\2/g' % rootdev.fs.uuid, '%s/boot/grub/menu.lst' % self.context.chroot_dir)
        run_cmd('sed', '-ie', 's/^# groot.*/# groot=%s/g' % bootdev.fs.uuid, '%s/boot/grub/menu.lst' % self.context.chroot_dir)
        run_cmd('sed', '-ie', '/^# kopt_2_6/ d', '%s/boot/grub/menu.lst' % self.context.chroot_dir)



#!/usr/bin/python
#
#    Uncomplicated VM Builder
#    Copyright (C) 2007-2009 Canonical Ltd.
#
#    See AUTHORS for list of contributors
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License version 3, as
#    published by the Free Software Foundation.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
#    The publically exposed bits of VMBuilder
#
import logging
import VMBuilder.log
import VMBuilder.plugins
from   VMBuilder.distro     import Distro
from   VMBuilder.hypervisor import Hypervisor
from   VMBuilder.plugins    import Plugin
from   VMBuilder.exception  import VMBuilderException, VMBuilderUserError

# Internal bookkeeping
distros = {}
hypervisors = {}
_distro_plugins = []
_hypervisor_plugins = []

# This is meant to be populated by plugins. It should contain a list of the files that we give back to the user.

def register_hypervisor(cls):
    """
    Register a hypervisor class with VMBuilder

    @type cls: Hypervisor
    @param cls: The new Hypervisor subclass to be registered with VMBuilder
    """
    hypervisors[cls.arg] = cls

def get_hypervisor(name):
    """
    Get Hypervisor subclass by name

    @type name: string
    @param name: Name of the Hypervisor subclass (defined by its .arg attribute)
    """
    if name in hypervisors:
        return hypervisors[name]
    else:
        raise VMBuilderUserError('No such hypervisor. Available hypervisors: %s' % (' '.join(hypervisors.keys())))

def register_distro(cls):
    """
    Register a distro class with VMBuilder

    @type cls: Distro
    @param cls: The new Distro subclass to be registered with VMBuilder
    """
    distros[cls.arg] = cls

def get_distro(name):
    """
    Get Distro subclass by name

    @type name: string
    @param name: Name of the Distro subclass (defined by its .arg attribute)
    """
    if name in distros:
        return distros[name]
    else:
        raise VMBuilderUserError('No such distro. Available distros: %s' % (' '.join(distros.keys())))

def register_distro_plugin(cls):
    """
    Register a distro plugin with VMBuilder

    B{Note}: A "distro plugin" is not a plugin that implements a new
    Distro.  It's a plugin that pertains to Distro's.  If you want to
    register a new Distro, use register_distro.

    @type cls: Plugin
    @param cls: The Plugin class to registered as a distro plugin
    """
    _distro_plugins.append(cls)
    _distro_plugins.sort(key=lambda x: x.priority)

def register_hypervisor_plugin(cls):
    """
    Register a hypervisor plugin with VMBuilder

    B{Note}: A "hypervisor plugin" is not a plugin that implements a new
    Hypervisor.  It's a plugin that pertains to Hypervisor's.  If you
    want to register a new Hypervisor, use register_hypervisor.

    @type cls: Plugin
    @param cls: The Plugin class to registered as a hypervisor plugin
    """
    _hypervisor_plugins.append(cls)
    _hypervisor_plugins.sort(key=lambda x: x.priority)

def set_console_loglevel(level):
    """
    Adjust the loglevel that will be sent to the console.

    @type level: number
    @param level: See the standard logging module
    """
    VMBuilder.log.console.setLevel(level)

def get_version_info():
    """
    Return a dict containing version information for VMBuilder.

    @return: A dict with (at least) the following keys:
             - major: Major version number.
             - minor: Minor version number.
             - micro: Micro version number.
             - revno: The revision number of the current branch or the branch from which the tarball was created.
    """
    import vcsversion
    info = vcsversion.version_info
    info['major'] = 0
    info['minor'] = 12
    info['micro'] = 4
    return info

logging.debug('Loading plugins')
VMBuilder.plugins.load_plugins()
